/*

VMEDIT, a general multipage GTK based text editor.

Copyright (C) 2005  Vladimir Martinek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <stdlib.h>
#include <libintl.h>
#include "vmedit.h"
#include "vmedit-uim.h"

extern int debug_level;

// Menu Layout
static const char * ui_description =
"<ui>"
"  <menubar name='MainMenu'>"
"    <menu action='FileMenu'>"
"      <menuitem action='New'/>"
"      <menuitem action='Open'/>"
"      <menuitem action='Save'/>"
"      <menuitem action='SaveAs'/>"
"      <separator/>"
"      <menuitem action='Close'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='EditMenu'>"
"      <menuitem action='Copy'/>"
"      <menuitem action='Cut'/>"
"      <menuitem action='Paste'/>"
"    </menu>"
"    <menu action='SearchMenu'>"
"      <menuitem action='Find'/>"
"      <menuitem action='Replace'/>"
"    </menu>"
"    <menu action='OptionsMenu'>"
"      <menuitem action='Wrap'/>"
"      <menuitem action='Status'/>"
"    </menu>"
"  </menubar>"
"  <toolbar name='MainToolbar'>"
"    <toolitem action='New'/>"
"    <separator/>"
"    <toolitem action='Copy'/>"
"    <toolitem action='Paste'/>"
"    <toolitem action='Cut'/>"
"  </toolbar>"
"</ui>";

// Normal items
static const GtkActionEntry ui_entries[] = {

  // Top Level
  { "FileMenu", NULL, "File" },
  { "EditMenu", NULL, "Edit" },
  { "OptionsMenu", NULL, "Options" },
  { "SearchMenu", NULL, "Search" },

  // File Menu Actions
  { "New",  GTK_STOCK_NEW,  "New",  "<control>N", "Create a New File",  create_new_file},
  { "Open", GTK_STOCK_OPEN, "Open", "<control>O", "Open File", open_file},
  { "Save", GTK_STOCK_SAVE, "Save", "<control>S", "Save File", save_file_menu},
  { "SaveAs", GTK_STOCK_SAVE_AS, "Save As...", NULL, "Save File As...", save_as_file_menu},
  { "Close", GTK_STOCK_CLOSE, "Close", NULL, "Close File", close_file},
  { "Quit", GTK_STOCK_QUIT, "Quit", "<control>Q", "Quit VMEdit", quit},

  // Edit Menu Actions
  { "Copy", GTK_STOCK_COPY, "Copy", "<control>C", "Copy to Clipboard", edit_copy},
  { "Cut", GTK_STOCK_CUT, "Cut", "<control>X", "Cut", edit_cut},
  { "Paste", GTK_STOCK_PASTE, "Paste", "<control>V", "Paste", edit_paste},

  // Search Menu Actions
  { "Find", GTK_STOCK_FIND, "Find", "<control>F", "Find", find},
  { "Replace", GTK_STOCK_FIND_AND_REPLACE, "Replace", "<control>R", "Replace", replace},

};

// Toggle items
static const GtkToggleActionEntry ui_toggles[] = {
  { "Wrap", NULL , "Wrap", NULL, "Wrap Lines", set_wrap, FALSE },
  { "Status", NULL , "Status Bar", NULL, "Status Bar", set_statusbar, FALSE }
};
      



vmeditUI * createUI(GtkWidget * window) {

  vmeditUI * newUI;

  // Create the menubar and toolbar

  newUI = malloc(sizeof(vmeditUI));

  initializeUIActions(newUI,window);
  initializeUIManager(newUI);
  initializeUIWidgets(newUI);
  return newUI;

}

void disposeUI(vmeditUI * uim) {
  
  free(uim);

}


void initializeUIActions(vmeditUI * uim, GtkWidget * window) {

  uim->uiActions = gtk_action_group_new ("MenuActions");

  gtk_action_group_set_translation_domain(uim->uiActions, "vmedit");
  bind_textdomain_codeset ("vmedit", "UTF-8");
  gtk_action_group_add_actions (uim->uiActions, ui_entries, G_N_ELEMENTS (ui_entries), window);
  gtk_action_group_add_toggle_actions (uim->uiActions, ui_toggles, G_N_ELEMENTS (ui_toggles), window);
  //gtk_action_group_set_translation_domain(uim->uiActions, NULL);
}


void initializeUIManager(vmeditUI * uim) {

  GError *error;

  uim->uiManager =  gtk_ui_manager_new();
  if (uim->uiManager == NULL) debug_fatal("Failed to create UI Manager.");

  gtk_ui_manager_insert_action_group (uim->uiManager, uim->uiActions, 0);


  if (!gtk_ui_manager_add_ui_from_string (uim->uiManager, ui_description, -1, &error))
    debug_fatal("Failed to create UI items.");
  
}

void initializeUIWidgets(vmeditUI * uim) {

  uim->menubar = gtk_ui_manager_get_widget (uim->uiManager, "/MainMenu");
  uim->toolbar = gtk_ui_manager_get_widget (uim->uiManager, "/MainToolbar");
  uim->accelerators = gtk_ui_manager_get_accel_group(uim->uiManager);
}


