/*

VMEDIT, a general multipage GTK based text editor.

Copyright (C) 2005  Vladimir Martinek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <string.h>
#ifdef GPEBUILD
#include <gpe/init.h>
#include <gpe/errorbox.h>
#include <gpe/pixmaps.h>
#include <gpe/picturebutton.h>
#include <gpe/question.h>
#include <gpe/spacing.h>
#endif
#include "vmedit.h"
#include "vmedit-uim.h"

#include <locale.h>
#include <libintl.h>

#define _(_x) gettext (_x)

int debug_level = 3;

vmeditUI * uimgr;
gboolean  pageChanging;
GtkWidget *notebook;
GtkWidget *window;
GtkWidget * toolbar;
GPtrArray *managedFiles;
gint      managedFilesCount;
GtkWidget *fileSelection;
GtkItemFactory *item_factory;
GtkWidget *optionsWrap;
GtkWidget *searchPanel;
GtkWidget *replacePanel;
GtkWidget *statusbar;
GtkWidget *findEntry;     // The edit item of the Find panel
GtkWidget *searchEntry;   // The search edit item of the Search/Replace panel
GtkWidget *replaceEntry;  // The replace edit item of the Search/Replace panel

gboolean searchPanelActive;
gboolean replacePanelActive;
gboolean statusbarActive;
int last_found = 0;

guint statusbarContext;


/* Returns a menubar widget made from the above menu */
static GtkWidget *get_menubar_menu( GtkWidget  *window )
{

  return uimgr->menubar;
}



/* You have to start somewhere */
int main( int argc,
          char *argv[] )
{

  GtkWidget *main_vbox;
  GtkWidget *menubar;
  


  /* Initialize GTK */

#ifdef GPEBUILD
  if (gpe_application_init (&argc, &argv) == FALSE)
    return -1;
#else
  gtk_init (&argc, &argv);
#endif


  /* Make a window */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  // Create UI Manager
  uimgr = createUI(window);

  g_signal_connect (G_OBJECT (window), "delete-event",
                    G_CALLBACK (quit),
                    NULL);
  gtk_window_set_title (GTK_WINDOW(window), _("Text Editor"));
  gtk_window_set_default_size(GTK_WINDOW(window), 200, 200);
 
  /* Make a vbox to put the three menus in */
  main_vbox = gtk_vbox_new (FALSE, 1);
  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 1);
  gtk_container_add (GTK_CONTAINER (window), main_vbox);
 
  /* Get the three types of menu */
  /* Note: all three menus are separately created, so they are not the
     same menu */
  menubar = get_menubar_menu (window);
  toolbar = get_toolbar();

  // Create notebook
  notebook = gtk_notebook_new();
  gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook),TRUE);
  
  /* Pack it all together */
  gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (main_vbox),toolbar, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (main_vbox),notebook , TRUE, TRUE, 0);

  g_signal_connect (G_OBJECT (notebook), "switch-page",
                    G_CALLBACK (page_change),
                    NULL);

  searchPanel = get_find_panel();
  searchPanelActive = FALSE;
  gtk_box_pack_start(GTK_BOX (main_vbox),searchPanel, FALSE, FALSE, 0);

  replacePanel = get_replace_panel();
  replacePanelActive = FALSE;
  gtk_box_pack_start(GTK_BOX (main_vbox),replacePanel, FALSE, FALSE, 0);

  statusbar = gtk_statusbar_new();
  statusbarContext = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),"Test");
  statusbarActive = FALSE;
  gtk_box_pack_end(GTK_BOX (main_vbox),statusbar, FALSE, FALSE, 0);
  // Create first page

  managedFiles = g_ptr_array_new();
  managedFilesCount=0;
  fileSelection = NULL;
  pageChanging = FALSE;

  // Load accelerators
  gtk_window_add_accel_group(GTK_WINDOW(window),uimgr->accelerators);
  //create_new_file();
  /* Show the widgets */
  gtk_widget_show(window);
  gtk_widget_show(main_vbox);
  gtk_widget_show_all(menubar);
  gtk_widget_show_all(toolbar);
  gtk_widget_show_all(notebook);


  /* Finished! */
  gtk_main ();

  disposeUI(uimgr);
 
  return 0;
}

void create_new_file(void) {

  GtkWidget *view;
  GtkWidget *tab_label, *tab_box;
  GtkWidget *tab_button, *tab_icon;
  GtkWidget *scrolledWindow;
  GtkTextBuffer *buffer; 
  GtkAdjustment *hadjust, *vadjust;
  ManagedFile * fileDetails;
  int pagenum;


  view = gtk_text_view_new ();
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));

  // Ensure status information update
  g_signal_connect_after (G_OBJECT(buffer),"mark-set",G_CALLBACK(update_cursor_position),buffer);


  gtk_text_buffer_set_text (buffer, "", -1);
  hadjust = GTK_ADJUSTMENT(gtk_adjustment_new(1.0,  -32000.0, 32000.0, 1.0, 5.0, 5.0));
  vadjust = GTK_ADJUSTMENT(gtk_adjustment_new(1.0,  -32000.0, 32000.0, 1.0, 5.0, 5.0));
  scrolledWindow=gtk_scrolled_window_new(hadjust,vadjust);
  gtk_container_add (GTK_CONTAINER (scrolledWindow), view);

  // Create widget for page tab
  tab_label = gtk_label_new (_("New File"));


  tab_button = gtk_button_new();
  gtk_button_set_relief(GTK_BUTTON(tab_button),GTK_RELIEF_NONE);
  tab_icon   = gtk_image_new_from_file(IMAGE_PATH "/vmedit-close.png");
  tab_box = gtk_hbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(tab_button),tab_icon);

  gtk_box_pack_start (GTK_BOX (tab_box), tab_label, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (tab_box), tab_button, FALSE, FALSE, 0);

  gtk_widget_show(tab_icon);
  gtk_widget_show(tab_button);
  gtk_widget_show_all(tab_box);
  
  // Add a new item to the array

  fileDetails = g_new0(ManagedFile,1);
  fileDetails->textView=view;
  fileDetails->fileName=NULL;
  fileDetails->labelItem = tab_label;
  g_ptr_array_add(managedFiles,fileDetails);
  managedFilesCount++;

  
  pagenum = gtk_notebook_append_page (GTK_NOTEBOOK (notebook), scrolledWindow, tab_box);
  gtk_widget_show_all(notebook);
  gtk_notebook_set_current_page(GTK_NOTEBOOK (notebook),pagenum);
  gtk_signal_connect_object (GTK_OBJECT (tab_button),"clicked",
			     GTK_SIGNAL_FUNC (do_close_page),fileDetails);

}

void edit_copy(void) {

  if (managedFilesCount <= 0) return; // all removed
  g_signal_emit_by_name(get_current_view(),"copy-clipboard");

}

void edit_paste(void) {
  if (managedFilesCount <= 0) return; // all removed
  g_signal_emit_by_name(get_current_view(),"paste-clipboard");

}

void edit_cut(void) {
  if (managedFilesCount <= 0) return; // all removed
  g_signal_emit_by_name(get_current_view(),"cut-clipboard");

}

void open_file(void) {

  fileSelection = gtk_file_selection_new(_("Open File"));

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fileSelection)->ok_button),
			     "clicked", GTK_SIGNAL_FUNC (do_open_file), NULL);
  
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fileSelection)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) fileSelection);
  gtk_window_set_transient_for(GTK_WINDOW(fileSelection), GTK_WINDOW(window));
  gtk_window_set_modal(GTK_WINDOW(fileSelection), TRUE);

  gtk_widget_show (fileSelection);
  
}

gboolean save_file(void) {

  ManagedFile * fileDetail;
  fileDetail = getFileDetail(get_current_page());
  
  if (fileDetail->fileName == NULL)
    return save_as_file();
  else
    return do_save_file(fileDetail->fileName);

}

gboolean save_as_file(void) {

  GtkWidget *fileSelection;
  gchar *fileName;
#ifndef GPEBUILD
  GtkWidget * messageDialog;
#endif
  gboolean doSave;
  gboolean res;

  res = FALSE;

  fileSelection = gtk_file_selection_new (_("Save As..."));
  gtk_window_set_transient_for(GTK_WINDOW(fileSelection), GTK_WINDOW(window));
  gtk_window_set_modal(GTK_WINDOW(fileSelection), TRUE);

  
  if (gtk_dialog_run (GTK_DIALOG (fileSelection)) == GTK_RESPONSE_OK) {
    
    fileName = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fileSelection)));

    // Check whether the file exists, if so, then raise a question

    doSave=FALSE;
    if (g_file_test(fileName,G_FILE_TEST_EXISTS)) {

#ifdef GPEBUILD
      
      doSave = gpe_question_ask (_("Current file is modified, save?"),_("Question"), 
				 "question", "!gtk-no", NULL, "!gtk-yes", NULL, NULL);
#else
      messageDialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,
					     GTK_BUTTONS_YES_NO,_("File %s already exists.\nOverwrite?"),fileName);
      doSave=(gtk_dialog_run (GTK_DIALOG (messageDialog))==GTK_RESPONSE_YES);
      gtk_widget_destroy (messageDialog);
#endif
    } else
      doSave = TRUE;


    if (doSave) {
      // Save file, if fails, free the fileName. If succeeded,
      // then leave it, will be freed when closing file.
      res = do_save_file(fileName);
      if (!res)
	g_free(fileName);
    } else
      g_free(fileName);

  }
  gtk_widget_destroy (fileSelection);
  return res;

}


gboolean do_open_file () {

  gchar *text, *utf8_text, *fileName;
  gsize length;
  GError *error = NULL;
  gboolean ret = FALSE;
  text=NULL;
  utf8_text=NULL;
  GtkTextBuffer * buffer;

  fileName = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fileSelection)));
  gtk_widget_destroy(fileSelection);
  if (g_file_get_contents (fileName, &text, &length, NULL)) {
    utf8_text = g_convert (text, length, "UTF-8", "ISO-8859-1", 
			   NULL, NULL, &error);
    if (error != NULL) {
      g_print(_("Error: Couldn't convert file %s to UTF-8\n"), fileName);
      g_error_free (error);
    }
    else {
      create_new_file();
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(get_current_view()));
      gtk_text_buffer_set_text(buffer,utf8_text,length);
      set_page_title(get_current_page(),basename(fileName));
      set_detail_file_name(get_current_page(),fileName);
      gtk_text_buffer_set_modified(buffer,FALSE);
      ret = TRUE;
    }
    
  }
  else 
    g_print(_("Error: Unable to read file %s\n"), fileName);

  if (text!=NULL) g_free(text);

  if (!ret) g_free(fileName);
  
  return ret;
}


gboolean do_save_file (gchar * fileName) {

  gchar *text = NULL;
  gchar *utf8_text = NULL;
  GError *error = NULL;
  GtkTextIter start, end;
  GtkTextBuffer *buffer;
  gsize length;
  gboolean ret = FALSE;
  FILE * fp;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(get_current_view()));

  gtk_text_buffer_get_bounds(buffer, &start, &end);

  utf8_text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);

  text = g_convert (utf8_text,-1,"ISO-8859-1","UTF-8",
		    NULL, NULL, &error);

  if (error != NULL) {
    g_print(error->message);
    g_error_free (error);
  }
  else {
    // Actual Write
    if ( (fp = fopen(fileName, "w")) == NULL)
      {
	g_print(_("Error: Couldn't open file %s for writing.\n"), fileName);
      } else {
	length = strlen(text);
        fwrite (text, 1, length, fp);
        fclose (fp);
	set_page_title(get_current_page(),basename(fileName));
	set_detail_file_name(get_current_page(),fileName);
	gtk_text_buffer_set_modified(buffer,FALSE);
	ret = TRUE;
      }
  }
  return ret;
}

void close_file(void) {

  guint pageNumber;


  if (managedFilesCount <= 0) return; // all removed
  
  pageNumber = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

  do_close_file(pageNumber);

}

void do_close_file(guint pageNumber) {

  ManagedFile * fileDetail;
  GtkResponseType resp;


  gtk_notebook_set_current_page(GTK_NOTEBOOK (notebook),pageNumber);
  fileDetail = getFileDetail(pageNumber);


  resp = ask_before_closing(pageNumber);
  if (resp == GTK_RESPONSE_CANCEL) return; // cowardly running away

  if (resp == GTK_RESPONSE_YES)
    if (!save_file()) return; // save file failed, return
      
  
  g_ptr_array_remove_index(managedFiles,pageNumber);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), pageNumber);
  if (fileDetail->fileName != NULL) g_free(fileDetail->fileName);
  managedFilesCount --;

}


ManagedFile * getFileDetail(guint pageNumber) {
  // find file detail for selected page number
  // Returns pointer to that detail or NULL if
  //not found

  ManagedFile * managedFile;

  if (pageNumber >= managedFilesCount) return NULL;
  
  managedFile = g_ptr_array_index (managedFiles,pageNumber);
  return managedFile;
}


void page_change (GtkNotebook *notebook,GtkNotebookPage *page,
		  guint page_num,gpointer user_data) {


  // Update page-depended options (wrap)
  pageChanging = TRUE;
  update_page_options(page_num);
  pageChanging = FALSE;
}

void set_page_title(guint pageNumber, gchar * pageTitle) {

  GtkWidget * labelItem;
  GtkWidget * currentPage;
  ManagedFile * fileDetail;


  fileDetail = getFileDetail(pageNumber);
  currentPage=gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),pageNumber);
  labelItem = fileDetail->labelItem;
  gtk_label_set_text(GTK_LABEL(labelItem),pageTitle);

}

GtkWidget * get_current_view() {

  guint pageNumber;

  ManagedFile * fileDetail;
  pageNumber = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
  
  fileDetail = getFileDetail(pageNumber);
  
  return fileDetail->textView;
  
}

guint get_current_page() {

  return gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

}

void set_detail_file_name(guint pageNumber, gchar * fileName) {


  ManagedFile * fileDetail;
  fileDetail = getFileDetail(pageNumber);

  if (fileDetail->fileName == fileName) return; // reassigned, nothing to do
  
  // release previously allocated file name
  if (fileDetail->fileName != NULL) g_free(fileDetail->fileName);
  fileDetail->fileName = fileName;
  
}

GtkResponseType ask_before_closing(guint pageNumber) {
  
  ManagedFile * fileDetail;
  GtkTextBuffer * buffer;
#ifndef GPEBUILD
  GtkWidget * messageDialog;
#endif
  GtkResponseType rsp;

  fileDetail = getFileDetail(pageNumber);

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(fileDetail->textView));
  rsp = GTK_RESPONSE_NO;


  if (gtk_text_buffer_get_modified(buffer)) {
    gtk_notebook_set_current_page(GTK_NOTEBOOK (notebook),pageNumber);
#ifdef GPEBUILD

    if (gpe_question_ask (_("Current file is modified, save?"),_("Question"), 
			  "question", "!gtk-no", NULL, "!gtk-yes", NULL, NULL))
      rsp = GTK_RESPONSE_YES;
#else
    
    
    messageDialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,
					   GTK_BUTTONS_NONE,
					   _("File is modified.Do you wish to save it first?"));
    gtk_dialog_add_button(GTK_DIALOG(messageDialog),_("Yes"),GTK_RESPONSE_YES);
    gtk_dialog_add_button(GTK_DIALOG(messageDialog),_("No"),GTK_RESPONSE_NO);
    gtk_dialog_add_button(GTK_DIALOG(messageDialog),_("Cancel"),GTK_RESPONSE_CANCEL);
    
    rsp = gtk_dialog_run (GTK_DIALOG (messageDialog));
    gtk_widget_destroy (messageDialog);

#endif

  }
  return rsp;

}

void save_as_file_menu(void) {
  
  if (managedFilesCount <= 0) return; // all removed
  save_as_file();
}

void save_file_menu(void) {
  if (managedFilesCount <= 0) return; // all removed
  save_file();
}

GtkWidget * get_toolbar() {

  return uimgr->toolbar;

}
void quit(void) {
  
  int i;
  GtkResponseType resp;
  for (i=0;i<managedFilesCount;i++) {
    resp = ask_before_closing(i);
    if (resp == GTK_RESPONSE_CANCEL) return; // cowardly running away
    
    if (resp == GTK_RESPONSE_YES)
      if (!save_file()) return; // save file failed, return
  }
  gtk_exit(0);
}

void do_close_page(ManagedFile * fileDetails) {

  do_close_file(get_page_number(fileDetails));

}

int get_page_number(ManagedFile * fileDetails) {

  int i;
  ManagedFile * managedFile;

  for (i=0;i<managedFilesCount;i++) {
    managedFile = g_ptr_array_index (managedFiles,i);
    if (managedFile == fileDetails) return i;
  }
  return -1;
}


void set_wrap () {

  ManagedFile * managedFile;
  gboolean wrap;
  GtkTextView * tw;

  if (pageChanging) return;

  if (managedFilesCount <= 0) return;

  managedFile = g_ptr_array_index (managedFiles,get_current_page());
  tw = GTK_TEXT_VIEW(managedFile->textView);
  wrap = (gtk_text_view_get_wrap_mode(GTK_TEXT_VIEW(managedFile->textView))!=GTK_WRAP_NONE);
  wrap = !wrap;
  if (wrap) gtk_text_view_set_wrap_mode(tw,GTK_WRAP_WORD);
  else gtk_text_view_set_wrap_mode(tw,GTK_WRAP_NONE);
  
  //g_print("Wrap set to %d for page %d\n",wrap,get_current_page());

  
}

void update_page_options(int pageNumber) {

  ManagedFile * managedFile;
  gboolean wrap;
  GtkTextView * tw;
  GtkTextBuffer * buffer;

  managedFile = g_ptr_array_index (managedFiles,pageNumber);

  // Update Wrap Toggle
  wrap = (gtk_text_view_get_wrap_mode(GTK_TEXT_VIEW(managedFile->textView))!=GTK_WRAP_NONE);

  // Update statusbar's row/column counter
  tw = GTK_TEXT_VIEW(managedFile->textView);
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tw));  
  update_cursor_position(buffer);  

}


GtkWidget * get_find_panel(void) {

  GtkWidget *hbox, *label, *button1, *button2, *icon;
  hbox = gtk_hbox_new (FALSE, 0);

  label = gtk_label_new (_("Search for: "));
  findEntry = gtk_entry_new ();

  button1 = gtk_button_new();
  gtk_button_set_relief(GTK_BUTTON(button1),GTK_RELIEF_NONE);
  icon = gtk_image_new_from_stock (GTK_STOCK_FIND, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button1),icon);
  g_signal_connect (G_OBJECT (button1), "clicked",G_CALLBACK (find_callback),NULL);


  button2 = gtk_button_new();
  gtk_button_set_relief(GTK_BUTTON(button2),GTK_RELIEF_NONE);
  icon   = gtk_image_new_from_file(IMAGE_PATH "/vmedit-close.png");
  gtk_container_add(GTK_CONTAINER(button2),icon);
  g_signal_connect (G_OBJECT (button2), "clicked",G_CALLBACK (find),NULL);


  
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), findEntry, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), button1, FALSE, FALSE, 0);
  gtk_box_pack_end (GTK_BOX (hbox), button2, FALSE, FALSE, 0);
  
  return hbox;

}


GtkWidget * get_replace_panel(void) {

  GtkWidget *table, *label1, *label2,
    *button1, *button2, *icon;
  
  label1 = gtk_label_new (_("Search for: "));
  gtk_label_set_justify(GTK_LABEL(label1),GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment(GTK_MISC(label1),0.0,0.0);
  label2 = gtk_label_new (_("Replace with: "));
  gtk_misc_set_alignment(GTK_MISC(label2),0.0,0.0);
  searchEntry = gtk_entry_new ();
  replaceEntry = gtk_entry_new ();
  button1 = gtk_button_new();
  gtk_button_set_relief(GTK_BUTTON(button1),GTK_RELIEF_NONE);
  icon   = gtk_image_new_from_file(IMAGE_PATH "/vmedit-close.png");
  gtk_container_add(GTK_CONTAINER(button1),icon);
  g_signal_connect (G_OBJECT (button1), "clicked",G_CALLBACK (replace),NULL);

  button2 = gtk_button_new();
  gtk_button_set_relief(GTK_BUTTON(button2),GTK_RELIEF_NONE);
  icon = gtk_image_new_from_stock (GTK_STOCK_FIND, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button2),icon);
  g_signal_connect (G_OBJECT (button2), "clicked",G_CALLBACK (replace_callback),NULL);


  table = gtk_table_new(3,2,FALSE);

  gtk_table_attach(GTK_TABLE(table),label1,0,1,0,1,GTK_FILL,GTK_SHRINK,2,0);
  gtk_table_attach(GTK_TABLE(table),searchEntry,1,2,0,1,GTK_FILL | GTK_EXPAND,GTK_SHRINK,2,0);
  gtk_table_attach(GTK_TABLE(table),button1,2,3,0,1,GTK_SHRINK,GTK_SHRINK,2,0);

  gtk_table_attach(GTK_TABLE(table),label2,0,1,1,2,GTK_FILL,GTK_SHRINK,2,0);
  gtk_table_attach(GTK_TABLE(table),replaceEntry,1,2,1,2,GTK_FILL | GTK_EXPAND,GTK_SHRINK,2,0);
  gtk_table_attach(GTK_TABLE(table),button2,2,3,1,2,GTK_SHRINK,GTK_SHRINK,2,0);




  return table;

}

void find(void) {

  if (searchPanelActive)
    gtk_widget_hide_all(searchPanel);
  else
    gtk_widget_show_all(searchPanel);
  
  searchPanelActive = !searchPanelActive;

}


void replace(void) {

  if (replacePanelActive)
    gtk_widget_hide_all(replacePanel);
  else
    gtk_widget_show_all(replacePanel);
  
  replacePanelActive = !replacePanelActive;

}

void set_statusbar() {

  if (statusbarActive)
    gtk_widget_hide_all(statusbar);
  else
    gtk_widget_show_all(statusbar);
  statusbarActive = !statusbarActive;
  
}


void find_callback() {
  ManagedFile * managedFile;
  GtkTextView * tw;
  gchar * searchText;
#ifndef GPEBUILD
  GtkWidget * messageDialog;
#endif
  if (managedFilesCount <= 0) return; // No files currently open
  managedFile = g_ptr_array_index (managedFiles,get_current_page());
  tw = GTK_TEXT_VIEW(managedFile->textView);  
  searchText = gtk_editable_get_chars (GTK_EDITABLE (findEntry), 0, -1);
  if (!do_find_string(tw,searchText)) {
#ifdef GPEBUILD
      gpe_error_box (_("Text not found"));
#else
      messageDialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,
					     GTK_BUTTONS_OK,_("Text Not Found."));
      gtk_dialog_run (GTK_DIALOG (messageDialog));
      gtk_widget_destroy (messageDialog);
#endif
      last_found = 0;
  }
   g_free(searchText);
  
}


void replace_callback() {
  ManagedFile * managedFile;
  GtkTextView * tw;
  gchar * searchText, *replaceText;
  GtkWidget * messageDialog;

  if (managedFilesCount <= 0) return; // No files currently open
  managedFile = g_ptr_array_index (managedFiles,get_current_page());
  tw = GTK_TEXT_VIEW(managedFile->textView);  
  searchText = gtk_editable_get_chars (GTK_EDITABLE (searchEntry), 0, -1);
  replaceText = gtk_editable_get_chars (GTK_EDITABLE (replaceEntry), 0, -1);
  if (!do_find_string(tw,searchText)) {
#ifdef GPEBUILD
      gpe_error_box (_("Text not found"));
#else
      messageDialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,
					     GTK_BUTTONS_OK,_("Text Not Found."));
      gtk_dialog_run (GTK_DIALOG (messageDialog));
      gtk_widget_destroy (messageDialog);
#endif
      last_found = 0;
  } else {
    //g_print("Found, doing the replace");
    do_replace_string(tw,replaceText);
  }
   g_free(searchText);
   g_free(replaceText);
  
}




gboolean do_find_string(GtkTextView * tw, gchar * find) {

  gchar  *found, *buffer;
  GtkTextBuffer *buf;
  GtkTextIter start, end, cursor;
  glong found_start;

  buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tw));
  gtk_text_buffer_get_bounds (buf, &start, &end);
  buffer = gtk_text_buffer_get_text (buf, &start, &end, FALSE);

  // Sort out where to start from
  gtk_text_buffer_get_iter_at_mark(buf,&cursor,gtk_text_buffer_get_insert(buf));
  last_found = gtk_text_iter_get_offset(&cursor);

  found = strstr (last_found + buffer + 1, find);

  if (!found) {
    g_free(buffer);
    return FALSE;
  }
  else
    {
      found_start = g_utf8_pointer_to_offset (buffer,found);
      //found_start = found - buffer;
      gtk_text_iter_set_offset (&start, found_start);
      gtk_text_iter_set_offset (&end, found_start + g_utf8_strlen(find,-1));
      
      gtk_text_buffer_move_mark_by_name (buf, "insert", &start);
      gtk_text_buffer_move_mark_by_name (buf, "selection_bound", &end);
      
      last_found = found_start;

      gtk_text_view_scroll_to_iter(tw,&start,0,TRUE,0.5,0.5);

    }

  g_free(buffer);
  return TRUE;

}

void do_replace_string (GtkTextView * tw, gchar * replace_with)
{

  GtkTextBuffer *buf; 
  GtkTextIter sel_start, sel_end, at;

  buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tw));

  if (gtk_text_buffer_get_selection_bounds (buf, &sel_start, &sel_end))
  {
      gtk_text_buffer_get_iter_at_offset (buf, &at, last_found);
      gtk_text_buffer_insert (buf, &at, replace_with, strlen(replace_with));
      gtk_text_buffer_delete_selection (buf, FALSE, TRUE);
      gtk_text_buffer_get_selection_bounds (buf, &sel_start, &sel_end);


      gtk_text_buffer_get_iter_at_offset (buf, &at, last_found);
      last_found = gtk_text_iter_get_offset (&sel_end);

      gtk_text_buffer_move_mark_by_name (buf, "insert", &at);
      gtk_text_iter_set_offset (&sel_end, gtk_text_iter_get_offset(&at)+g_utf8_strlen(replace_with,-1));
      gtk_text_buffer_move_mark_by_name (buf, "selection_bound", &sel_end);
  }
}

void update_cursor_position(GtkTextBuffer *buffer) {

  GtkTextIter cursor;
  gint row,col;
  gchar message[32];
  if (statusbarActive) {
    gtk_text_buffer_get_iter_at_mark(buffer,&cursor,gtk_text_buffer_get_insert(buffer));
    row = gtk_text_iter_get_line(&cursor)+1;
    col = gtk_text_iter_get_line_offset(&cursor)+1;
    snprintf(message,32,"Row: %d, Column: %d",row,col);
    gtk_statusbar_pop(GTK_STATUSBAR(statusbar),statusbarContext);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar),statusbarContext,message);

  }
}

/* TODO
1) Find/Find Next does not find the word if cursor
   stands at the begining of it. UTF-8 related positioning
   problems,
2) Default buttons - pressing enter in the find/search/replace
   should launch the action
3) Encoding selection on file open/save, so far it's only ISO-8859-1
*/
